/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "driver_api.hh"
#include "seg_pool_mgr.hh"

#include "segdb.hh"
#include "chan_list.hh"
#include "user_data.hh"
#include <iostream>

using namespace sends;
using namespace std;

//======================================  Construct a driver_api
driver_api::driver_api(void) 
    : mDebug(0), mEnable(1)
{
}

//======================================  Destroy a driver_api
driver_api::~driver_api(void) {
}

//======================================  Get segments for a list of channels
int 
driver_api::get_segments(gps_type start, gps_type end, const channel_vect& cv,
			 segment_vect& sv) 
{
    //----------------------------------  Initialize the segment list.
    int N = cv.size();
    sv.clear();
    sv.resize(N, Segment_List.end());

    //----------------------------------  Get the segments in advance
    index_type nSeg = 0;
    for (int i=0; i<N; ++i) {
	chan_index  ichn = cv[i];
	double      rate = Channel_List[ichn].rate();
	int        wsize = Channel_List[ichn].data_size();
	gps_type   pfill = start;
	while (pfill < end) {
	    if (nSeg == sv.size()) sv.push_back(Segment_List.end());
	    sv[nSeg] = Segment_List.get_buffer(Buffer_Pool, ichn, rate, wsize,
					       pfill, end);
	    seg_index iSeg = sv[nSeg];
	    if (Segment_List.is_null(sv[nSeg])) break;

	    //--------------------------  Release segment if the data is there
	    if (Segment_List[iSeg].getEndGPS() >= end) {
		release_segment(iSeg);
		sv[nSeg] = Segment_List.end();
		break;
	    }
	    gps_type nxfill = Segment_List[iSeg].getReserveGPS();
	    if (debug() >= 3) {
		cerr << "Allocated segment: " << iSeg << " for channel: "
		     << Channel_List[ichn].name() << " between " 
		     << pfill << "-" << nxfill << endl;
		if (!nSeg) segment::dump_header(cerr);
		Segment_List[iSeg].dump(cerr);
	    }
	    nSeg++;
	    pfill = nxfill;
	}
    }
    return nSeg;
}

//======================================  Default current time - not-online
gps_type 
driver_api::online_gps(void) {
    return 0;
}

//======================================  Tell the driver to read a channel
int 
driver_api::release_channel(chan_index c) {
    return 0;
}

//======================================  Release segments in list
void 
driver_api::release_segments(segment_vect& sv) {
    index_type N = sv.size();
    for (index_type i=0; i<N; ++i) {
	seg_index iseg = sv[i];
	release_segment(iseg);
	sv[i] = Segment_List.end();
    }
}

//======================================  Release a single segment
void 
driver_api::release_segment(seg_index iseg) const {
    if (!Segment_List.is_null(iseg)) {
	Segment_List[iseg].fill_unlock();
	Segment_List.release(iseg);
    }
}

//======================================  Tell the driver to forget a channel
int 
driver_api::request_channel(chan_index channel) {
    return 0;
}

//======================================  Set the access mask
void
driver_api::set_access(access_mask msk) {
    mEnable = msk;
}

//======================================  Set the debug printout level
void
driver_api::setDebug(int lvl) {
    mDebug = lvl;
}

//======================================  Test the user access
access_mask
driver_api::test_access(const user_data& user) {
    return (mEnable & user.access());
}
